@@ -20,6 +20,8 @@
 #include "dfi_mem_chunk.h"
 #include "dfi_vmcoreinfo.h"
 
+#include "boot/os_info.h"
+
 #ifdef __s390x__
 #define LC_VMCORE_INFO		0xe0c
 #else
@@ -27,26 +29,14 @@
 #endif
 
 #define LC_OS_INFO		0xe18
-#define OS_INFO_MAGIC		0x4f53494e464f535aULL /* OSINFOSZ */
 
-struct os_info {
-	u64	magic;
-	u32	csum;
-	u16	version_major;
-	u16	version_minor;
-	u64	crashkernel_addr;
-	u64	crashkernel_size;
-	u64	vmcoreinfo_addr;
-	u64	vmcoreinfo_size;
-	u32	vmcoreinfo_csum;
-	u64	reipl_block_addr;
-	u64	reipl_block_size;
-	u32	reipl_block_csum;
-	u64	init_fn_addr;
-	u64	init_fn_size;
-	u32	init_fn_csum;
-	u8	reserved[4004];
-} __packed;
+struct vm_info {
+	u64	identity_base;
+	u64	kaslr_offset;
+	u64	kaslr_offset_phys;
+	u64	amode31_start;
+	u64	amode31_end;
+};
 
 /*
  * File local static data
@@ -54,6 +44,7 @@ struct os_info {
 static struct {
 	char		*vmcoreinfo;
 	struct os_info	*os_info;
+	struct vm_info  *vm_info;
 } l;
 
 static u32 os_info_csum(struct os_info *os_info)
@@ -71,6 +62,7 @@ static struct os_info *os_info_get(void)
 
 	if (dfi_mem_virt_read(LC_OS_INFO, &addr, sizeof(addr)))
 		return NULL;
+	util_log_print(UTIL_LOG_DEBUG, "DFI osinfo addr: 0x%016lx\n", addr);
 	if (addr % 0x1000)
 		return NULL;
 	if (dfi_mem_virt_read(addr, &os_info, sizeof(os_info)))
@@ -79,9 +71,46 @@ static struct os_info *os_info_get(void)
 		return NULL;
 	if (os_info.csum != os_info_csum(&os_info))
 		return NULL;
+
+	util_log_print(UTIL_LOG_DEBUG, "DFI found valid osinfo!\n");
 	return &os_info;
 }
 
+static struct vm_info *vm_info_get(void)
+{
+	static struct vm_info vm_info = { 0 };
+
+	util_log_print(UTIL_LOG_TRACE, "DFI read vm_info\n");
+
+	vm_info.identity_base = l.os_info->entry[OS_INFO_IDENTITY_BASE].val;
+	if (vm_info.identity_base == 0)
+		return NULL;
+	vm_info.kaslr_offset = l.os_info->entry[OS_INFO_KASLR_OFFSET].val;
+	vm_info.kaslr_offset_phys = l.os_info->entry[OS_INFO_KASLR_OFF_PHYS].val;
+	vm_info.amode31_start = l.os_info->entry[OS_INFO_AMODE31_START].val;
+	vm_info.amode31_end = l.os_info->entry[OS_INFO_AMODE31_END].val;
+
+	return &vm_info;
+}
+
+/*
+ * Convert virual address in the dump to the physical address using
+ * vm_info data derived from os_info
+ */
+u64 dfi_vm_vtop(u64 vaddr)
+{
+	if (!l.vm_info)
+		return vaddr;
+	if (vaddr < LOWCORE_SIZE)
+		return vaddr;
+	if ((vaddr < l.vm_info->amode31_end) &&
+	    (vaddr >= l.vm_info->amode31_start))
+		return vaddr;
+	if (vaddr < l.vm_info->kaslr_offset)
+		return vaddr - l.vm_info->identity_base;
+	return vaddr - l.vm_info->kaslr_offset + l.vm_info->kaslr_offset_phys;
+}
+
 /*
  * Initialize vmcoreinfo
  */
@@ -94,11 +123,13 @@ void dfi_vmcoreinfo_init(void)
 	util_log_print(UTIL_LOG_TRACE, "DFI vmcoreinfo initialization\n");
 
 	l.os_info = os_info_get();
+	if (l.os_info)
+		l.vm_info = vm_info_get();
 
-	if (l.os_info && l.os_info->vmcoreinfo_size) {
+	if (l.os_info && l.os_info->entry[OS_INFO_VMCOREINFO].size) {
 		util_log_print(UTIL_LOG_DEBUG, "DFI found valid osinfo\n");
-		addr = l.os_info->vmcoreinfo_addr;
-		size = l.os_info->vmcoreinfo_size;
+		addr = l.os_info->entry[OS_INFO_VMCOREINFO].addr;
+		size = l.os_info->entry[OS_INFO_VMCOREINFO].size;
 	} else {
 		if (dfi_mem_virt_read(LC_VMCORE_INFO, &addr, sizeof(addr)))
 			return;
@@ -175,6 +206,7 @@ static int vmcoreinfo_item_ulong(unsigned long *val, const char *fmt,
 	int rc;
 
 	rc = vmcoreinfo_item(str, sizeof(str), fmt, sym);
+	util_log_print(UTIL_LOG_DEBUG, "DFI vmcoreinfo symbol %s : %s\n", sym, str);
 	if (rc)
 		return rc;
 	*val = strtoul(str, NULL, base);
