@@ -370,7 +370,8 @@ get_command_line(int argc, char* argv[], struct command_line* line)
 }
 
 
-void free_target_data(struct job_target_data *data)
+static void
+free_target_data(struct job_target_data* data)
 {
 	free(data->bootmap_dir);
 	free(data->targetbase);
